use clap::{Parser, Subcommand, ValueEnum};

#[derive(Parser)]
#[clap(arg_required_else_help = true, verbatim_doc_comment, author)]
#[command(name = "PluralSync")]
#[command(author = "jvnkyard <lafresita@protonmail.com>")]
#[command(version = "1.2")]
#[command(
    help_template = "{name} Version {version} \n {author} \n {about-section} \n {usage-heading} {usage} \n \n {all-args} {tab}"
)]
#[command(about, long_about = None)]
/// Tool written in Rust to syncronize API calls between pluralkit and SimplyPlural.
/// Sets avatars for discord and fediverse.
pub struct Args {
    /// Command to execute
    #[command(subcommand)]
    pub cmd: Commands,
}

#[derive(Subcommand)]
pub enum Commands {
    ///Fetches the system member information from PluralKit and SimplyPlural
    Sync,
    /// Sets one or more members to the front
    #[clap(arg_required_else_help = true)]
    Set {
        /// Members to set
        members: Vec<String>,
        /// Overrides the configuration file to run the discord module
        #[cfg(feature = "discord")]
        #[arg[short, long]]
        discord: bool,
        /// Overrides the configuration file to run the fedi module
        #[cfg(feature = "fedi")]
        #[arg[short, long]]
        fedi: bool,
    },
    /// Refreshes the current front information
    Get {
        /// Selects which front takes priority in case of mismatch
        #[arg(short = 'o',long)]
        force_from: Option<ForceFrom>,
        /// Overrides the configuration file to run the discord module
        #[cfg(feature = "discord")]
        #[arg[short, long]]
        discord: bool,
        /// Overrides the configuration file to run the fedi module
        #[cfg(feature = "fedi")]
        #[arg[short, long]]
        fedi: bool,
    },
    /// Appends a member to the existing front
    #[clap(arg_required_else_help = true)]
    Add {
        /// Members to add
        members: Vec<String>,
        /// Overrides the configuration file to run the discord module
        #[cfg(feature = "discord")]
        #[arg[short, long]]
        discord: bool,
        /// Overrides the configuration file to run the fedi module
        #[cfg(feature = "fedi")]
        #[arg[short, long]]
        fedi: bool,
    },
    /// Displays a list with all members
    Members,
}

#[derive(ValueEnum, Clone)]
pub enum ForceFrom {
    SP,
    PK,
    None,
}
